'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var map = require('rxjs/operator/map');
var filter = require('rxjs/operator/filter');
var ignoreElements = require('rxjs/operator/ignoreElements');
var reduxSubspace = require('redux-subspace');
var reduxObservable = require('redux-observable');

var SUBSPACE_STORE_KEY = '@@redux-subspace/store';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};





















var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};













var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

/**
 * Copyright 2017, IOOF Holdings Limited.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree.
 */
var identity = function identity(x) {
    return x;
};

var subspaced = function subspaced(mapState, namespace) {
    var subspaceDecorator = reduxSubspace.subspace(mapState, namespace);
    return function (epic) {
        return function (action$, store) {
            var _context;

            var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

            var parentStore = _ref[SUBSPACE_STORE_KEY],
                dependencies = objectWithoutProperties(_ref, [SUBSPACE_STORE_KEY]);

            if (parentStore === undefined) {
                throw new Error('Subspace epic couldn\'t find the store. Make sure you\'ve used createEpicMiddleware from redux-subspace-observable');
            }
            var subspacedStore = subspaceDecorator(parentStore);

            Object.defineProperty(dependencies, SUBSPACE_STORE_KEY, {
                enumerable: false,
                configurable: false,
                writable: false,
                value: subspacedStore
            });

            var filteredAction$ = (_context = map.map.call(action$, function (action) {
                return subspacedStore.processAction(action, identity);
            }), filter.filter).call(_context, identity);

            return (_context = (_context = epic(filteredAction$, subspacedStore, dependencies), map.map).call(_context, subspacedStore.dispatch), ignoreElements.ignoreElements).call(_context);
        };
    };
};

var createEpicMiddleware = function createEpicMiddleware(rootEpic) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    if (options.dependencies != null && (Array.isArray(options.dependencies) || _typeof(options.dependencies) !== 'object')) {
        throw new TypeError('dependencies must be an object');
    }

    return reduxSubspace.applyToRoot(function (store) {
        var _babelHelpers$extends;

        var optionsWithStore = _extends({}, options, {
            dependencies: _extends({}, options.dependencies, (_babelHelpers$extends = {}, _babelHelpers$extends[SUBSPACE_STORE_KEY] = store, _babelHelpers$extends))
        });
        return reduxObservable.createEpicMiddleware(rootEpic, optionsWithStore)(store);
    });
};

/**
 * Copyright 2017, IOOF Holdings Limited.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree.
 */

exports.subspaced = subspaced;
exports.createEpicMiddleware = createEpicMiddleware;
